/* This file is Copyright 2002 Level Control Systems.  See the included LICENSE.txt file for details. */

#include <new>
#include <typeinfo>
#include <string.h>

#include "system/GlobalMemoryAllocator.h"

namespace muscle {

static MemoryAllocatorRef _globalAllocatorRef;

void SetCPlusPlusGlobalMemoryAllocator(MemoryAllocatorRef maRef)
{
   MemoryAllocatorRef holdPrev = _globalAllocatorRef;  // necessary to ensure correct order of operations!
   if (_globalAllocatorRef != maRef)
   {
      // Gotta do some special juju here so that _globalAllocatorRef's destructor
      // doesn't cause the referenced object to be deleted, because then it
      // will try to delete itself and crash.  So better just to leave it
      // allocated and let the system clean it up.
      if (_globalAllocatorRef()) _globalAllocatorRef()->DecrementRefCount();
      _globalAllocatorRef = maRef;
      if (_globalAllocatorRef()) _globalAllocatorRef()->IncrementRefCount();
   }
}

MemoryAllocatorRef GetCPlusPlusGlobalMemoryAllocator()
{
   return _globalAllocatorRef;
}

};  // end namespace muscle

// Metrowerk's compiler crashes at run-time if this memory-tracking
// code is enabled.  I haven't been able to figure out why (everything
// looks okay, but after a few dozen malloc()/free() calls, free()
// crashes!) so I'm just going to disable this code for PowerPC/Metrowerks
// machines.  Sorry!   --jaf 12/01/00
#ifdef __MWERKS__
# define DISABLE_MUSCLE_MEMORY_TRACKING
#endif

#ifndef DISABLE_MUSCLE_MEMORY_TRACKING

// VC++ doesn't know from exceptions  :^P
#ifdef WIN32
# ifndef MUSCLE_NO_EXCEPTIONS
#  define MUSCLE_NO_EXCEPTIONS
# endif
#endif

// No exceptions?  Then make all of these keywords go away
#ifdef MUSCLE_NO_EXCEPTIONS
# define BAD_ALLOC
# define THROW
# define LPAREN
# define RPAREN
#else
# define BAD_ALLOC bad_alloc
# define THROW     throw
# define LPAREN    (
# define RPAREN    )
#endif

void * operator new(size_t s) THROW LPAREN BAD_ALLOC RPAREN
{
   using namespace muscle;
   void * ret = (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
   if (ret == NULL) { THROW BAD_ALLOC LPAREN RPAREN; }
   return ret;
}

void * operator new[](size_t s) THROW LPAREN BAD_ALLOC RPAREN
{
   using namespace muscle;
   void * ret = (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
   if (ret == NULL) { THROW BAD_ALLOC LPAREN RPAREN; }
   return ret;
}

// Borland and VC++ don't like separate throw/no-throw operators, it seems
#ifndef WIN32

void * operator new(size_t s, nothrow_t const &) THROW LPAREN RPAREN
{
   using namespace muscle;
   return (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
}

void * operator new[](size_t s, nothrow_t const &) THROW LPAREN RPAREN
{
   using namespace muscle;
   return (_globalAllocatorRef()) ? _globalAllocatorRef()->Malloc(s) : malloc(s);
}

#endif

void operator delete(void * p) THROW LPAREN RPAREN
{
   using namespace muscle;
   if (p) 
   {
      if (_globalAllocatorRef()) _globalAllocatorRef()->Free(p);
                       else free(p);
   }
}

void operator delete[](void * p) THROW LPAREN RPAREN
{
   using namespace muscle;
   if (p) 
   {
      if (_globalAllocatorRef()) _globalAllocatorRef()->Free(p);
                            else free(p);
   }
}

#endif
